%------------------------------------------------------------------------
% params.pl : parameters for the solver
%
% version 2.0, 2003-2008
%
% Linh Anh Nguyen
% Institute of Informatics
% Warsaw University
% nguyen@mimuw.edu.pl
%
%------------------------------------------------------------------------

/*=======================================================================
Options:

- limit on length of a modality
	(limit_modality_length, Calculus, Limit)

- limits on number of rSat/rNF rules called in a resolution cycle
	(limit_rSat, Calculus, Limit)
	(limit_rNF, Calculus, Limit)

- limit on term depth
	(limit_term_depth, Limit)

- limit on proof size, not counting classical fragments and negations
	(limit_proof_size, Limit)

- deepening step for iterative deepening search w.r.t. the proof size
	(deepening_step_for_proof_size, Number)

- limit on the number of applications of rules
	(limit_rule_applications, Limit)

- deepening step for iterative deepening search w.r.t. the number of
  applications of rSat/rNF rules
	(deepening_step_for_rule_applications, Number)

%-----------------------------------------------------------------------%

- random selection of rSat/rNF rules
	(random_selection_of_rules, true/false)

- sorting modal clauses according to their body lengths
	(sorting_mclauses, true/false)

- use calling history
	(use_calling_history, Calculus, false/true)

- checking is divided into pre-check and post-check
	(check_in_two_steps, Calculus, true/false)

- is it allowed to create a copy of the calculus with other options?
	(copiable, Calculus, true/false)

%-----------------------------------------------------------------------%

- priority list of modal calculi for solving dum predicates
	(priority_list_of_calculi, ListOfCalculi)

- current calculus (for solving goals)
	(current_calculus, Calculus)

- save program before consulting files
	(save_before_consulting, true/false)

- debug mode
	(debug, false/true)

- quiet mode
	(quiet, true/false)

- show duplicates of answers
	(show_answer_duplicates, false/true)

- SICStus-Prolog
	(sicstus, true/false)

=======================================================================*/

:- set_option(limit_modality_length, 4).
:- set_option(limit_rSat, 3).
:- set_option(limit_rNF, 1).
:- set_option(limit_term_depth, 5). 
:- set_option(limit_proof_size, 1000000).
:- set_option(deepening_step_for_proof_size, 50).
:- set_option(limit_rule_applications, 1000000).
:- set_option(deepening_step_for_rule_applications, 30).

:- set_option(random_selection_of_rules, false).
:- set_option(sorting_mclauses, false).
:- set_option(use_calling_history, false).
:- set_option(check_in_two_steps, false).
:- set_option(copiable, false).

:- set_option(current_calculus, classical).
:- set_option(save_before_consulting, false).
:- set_option(debug, false).
:- set_option(quiet, true).
:- set_option(show_answer_duplicates, false).
:- set_option(sicstus, true).

%------------------------------------------------------------------------

:- dynamic universal_modal_operator/2. % (Calculus, Oper)
:- dynamic dual_modal_operator/3. % (Calculus, Oper, DualOper)
:- dynamic box_lifting_form/3. % (Calculus, Oper, BoxLiftingForm)
:- dynamic forward_labeled_form/3. % (Calculus, SimpleModalAtom, FLForm)

% if (check_in_two_steps, Calculus, false)
:- dynamic normal_labeled_form/2. % (Calculus, Modality)
:- dynamic operator_instance/3. % (Calculus, Instance, ModalOperator)
:- dynamic context_instance/3. % (Calculus, UniModality, Context)
% else
:- dynamic pre_check_normal_labeled_form/2.  
:- dynamic post_check_normal_labeled_form/2.  
:- dynamic pre_check_operator_instance/3. 
:- dynamic post_check_operator_instance/3. 
:- dynamic pre_check_context_instance/3. 
:- dynamic post_check_context_instance/3. 

%------------------------------------------------------------------------
