%------------------------------------------------------------------------
% parser.pl
%
% version 2.0, 2003-2008
%
% Linh Anh Nguyen 
% Institute of Informatics
% Warsaw University
% nguyen@mimuw.edu.pl
%
%------------------------------------------------------------------------

%========================================================================
% consult_cal(Files)
% mconsult_prog(Files, Calculus)
%========================================================================

:- op(1150, fx, pre_rSat). 
:- op(1150, fx, rSat).
:- op(1150, fx, post_rSat).
:- op(1150, fx, rNF).
:- op(1150, fx, calculus).
:- op(1150, fx, end).
:- op(1150, fx, dum_pred).
:- op(1200, xfx, ::).

%------------------------------------------------------------------------

peek_code_(Stream, Code) :- 
	at_end_of_stream(Stream) -> Code = -1; peek_code(Stream, Code).

%------------------------------------------------------------------------

parse_sep(S) :- 
	peek_code_(S, C), 
	memberchk(C, [32, 9, 10]), 
	get_code(S, _).

parse_seps(S) :-
	parse_sep(S), !, 
	parse_seps(S).

parse_seps(_).

parse_comment(S) :-
	parse_seps(S), 
	peek_code_(S, C), [C] = "%", 
	repeat, 
	  peek_code_(S, C2), 
	  (C2 \== -1 -> get_code(S, _); true),
	  (C2 = 10; C2 = -1), 
	!.

parse_comment(S) :-
	parse_seps(S), 
	peek_code_(S, C), [C] = "/", get_code(S, _), 
	get_code(S, C3), [C3] = "*",
	repeat,
	  get_code(S, C4), 
	  (  ([C4] = "*", peek_code_(S, C5), [C5] = "/", get_code(S, _))
	   ; C4 = -1),
	!,
	line_count(S, L), 
	check_not(C4 = -1, ['Line ', L, ' : expected end of comment.']).

parse_comments(S) :-
	parse_comment(S), !,  
	parse_comments(S).

parse_comments(_).

%------------------------------------------------------------------------

get_elem(S, Elem, Type) :-
	parse_comments(S), 
	peek_code_(S, C), C \== -1, !, 
	read(S, Elem), 
	functor(Elem, Func, Arity),	
	(Func = '::' -> Type = rule; 
	 ((Func = ':-'), (Arity = 1)) ->
	   (arg(1, Elem, Arg1), functor(Arg1, Type, _));
	 Type = clause_or_rule),
	!.
		
get_elem(S, end_of_file, end_of_file) :-
	peek_code_(S, -1).

peek_type(S, Type) :-
	parse_comments(S), 
	peek_code_(S, C),
	(C = -1 -> Type = end_of_file;
	 [C] = ":" -> Type = declaration;
	 Type = clause_or_rule).
	
%------------------------------------------------------------------------

declare(Decl) :- 
	get_option(sicstus, false) ->
		(Decl =.. [':-', Decl2], Decl2);
	(open('temp.pl', write, S) ->
           TempFileName = 'temp.pl'; 
	   get_option(mprolog_home, HOME),
	   name(HOME, Dir),
	   append(Dir, "/temp.pl", Name),
	   name(TempFileName, Name), 
	   open(TempFileName, write, S)), 
	write(S, Decl), write(S, '.'),
	flush_output(S), 
	close(S), 
	asserta(portray_message(informational, _)),
	consult(TempFileName), 
	retract(portray_message(informational, _)).

%------------------------------------------------------------------------

consult_cal(File) :- 
	atom(File), 
	open(File, read, S), 
	repeat,
	  get_elem(S, Elem, Type), 
	  (Type = end_of_file -> true; 
	  ((Type = end) -> true; 
	  (Type = op -> Elem;
	  (Type = include -> 
	     (Elem =.. [':-', include(Files)], 
	      consult_cal(Files)); 
	  (Type = rule -> 
	     (Elem =.. ['::', Name, Rule], 
	      dec_mrule(Name, Rule)); 
	  ((rule_cats(Cats), memberchk(Type, Cats)) -> 
	     (Elem =.. [':-', TL], 
	      TL =.. [Type, Calculi], 
	      read_mrules(S, Calculi, Type));
	  (Type \== clause_or_rule ->  
	     declare(Elem);
	  (assertz(Elem) -> true; 
	  line_count(S, Line), 
	  print_msgs(error, ['File ', File, ', line ', Line, 
		' : expected a clause.']),   
	  !, fail
	  )))))))),
	  Type = end_of_file, !, 
	close(S).

consult_cal([]).

consult_cal([File | Others]) :-
	consult_cal(File),
	consult_cal(Others).
	
read_mrules(S, Calculi, Cat) :- 
	to_list(Calculi, ListOfCalculi),
	get_list_of_calculi(ListOfCalculi2),  
	forall(Cal, ListOfCalculi, 
	       (\+ memberchk(Cal,ListOfCalculi2) -> dec_calculus(Cal); true)),
	repeat,
	  peek_type(S, Type), 
	  ((Type \== clause_or_rule, Type \== rule) -> true;  
	   get_elem(S, Rule, _),
	   (Rule =.. ['::', Name, RuleContent] -> true; 
	    (gen_rule_name(Name), RuleContent = Rule)),
	   dec_mrule(Name, RuleContent), 
	   forall(Cal2, ListOfCalculi, 
	          assertz_mrule(Cal2, Cat, Name))
	  ),
	  Type \== clause_or_rule, Type \== rule, 
	!.

%------------------------------------------------------------------------

mconsult_prog(File, Cal) :- 
	atom(File), 
	open(File, read, S),
	mconsult_prog_s(File, S, [Cal]),
	close(S).
 
mconsult_prog([], _).

mconsult_prog([File | Others], Cal) :-
	mconsult_prog(File, Cal),
	mconsult_prog(Others, Cal).

mconsult_prog_s(File, S, Calculi) :- 
	repeat,
	get_elem(S, Elem, Type), 
	(Type = end_of_file -> true; 
	(Type = op -> Elem;
	(Type = include -> 
	   (Elem =.. [':-', include(Files)], 
	    mconsult_prog(Files, Calculi));
	((Type = dum_pred) -> 
	   (Elem =.. [':-', dum_pred(Preds)],
	    to_list(Preds, ListOfPreds), 
	    dec_preds(dum, ListOfPreds)); 
	((Type = calculus) ->
	   (Elem =.. [':-', TL],
	    TL =.. [Type, Cals],    
	    to_list(Cals, Calculi2), 
	    get_list_of_calculi(ListOfCalculi),
	    for(Cal, Calculi2, 
	        (memberchk(Cal, ListOfCalculi) -> true; 
	         print_msgs(warning, [Type, ' ', Cal, ' was not declared.']))),   
	    mconsult_prog_s(File, S, Calculi2)); 
	((Type = end) ->
	    mconsult_prog_s(File, S, [classical]); 
	(Type \== clause_or_rule ->
	   declare(Elem);
	(Calculi = [classical] ->
	   (assertz(Elem) -> true; 
	    line_count(S, Line), 
	    print_msgs([error, 'File ', File, ', line ', Line,
		' : expected a classical clause.']),   
	    !, fail);
	gen_clause_name(Name),
	Calculi = [Cal | _], 
	break_clause(Cal, Elem, Context, Head, Body),  
	dec_mclause(Name, Context, Head, Body),
	get_pred(Head, Pred),  
	(get_option(sorting_mclauses, true) ->
	   AssertMethod = s;
	   AssertMethod = z),
	forall(Cal2, Calculi, 
	       (assert_mclause(AssertMethod, Cal2, Pred, Name) -> true; 
	        line_count(S, Line), 
	        print_msgs(error, ['File ', File, ', line ', Line,
			' : expected a modal clause.']),   
	        !, fail))
	)))))))),
	(Type = end_of_file; (Type = calculus)), !.

%------------------------------------------------------------------------

