%------------------------------------------------------------------------
% solver.pl
%
% version 2.0, 2003-2008
%
% Linh Anh Nguyen 
% Institute of Informatics
% Warsaw University
% nguyen@mimuw.edu.pl
%
%------------------------------------------------------------------------

:- asserta(portray_message(warning, clauses_not_together(_))).

%------------------------------------------------------------------------

mcall_(Cal, Goal) :- 
	set_info(mtrace_step, 0),
	set_info(already_called_mtrace, false),
	set_info(number_of_trials, 0),
	reset_tabulation, 

	getTime(StartTime),
	get_tabulation_size(TabSize0),

	StatusIn = [size/1, total_rSat/0, total_rSat_steps/0, 
			total_rNF/0, total_rNF_steps/0], 

	(
	 mcall_(Cal, Goal, StatusIn, StatusOut), 

	 (get_option(show_answer_duplicates, true) -> true;
	  ground_form(Goal, Answer),
	  \+ answer(Answer),
	  assertz(answer(Answer))), 

	 get_info(number_of_trials, N),
	 getTime(Now), Duration is Now - StartTime, 
	 StatusOut = [size/FoundSize, total_rSat/W2, total_rSat_steps/W3, 
			total_rNF/W4, total_rNF_steps/W5], 

	 print_msgs(debug, ['Trials of rules: ', N]),
	 print_msgs(debug, ['Applications of rSat: ', W2]),
	 print_msgs(debug, ['Applications of rSat rules: ', W3]),
	 print_msgs(debug, ['Applications of rNF: ', W4]),
	 print_msgs(debug, ['Applications of rNF rules: ', W5]),
	 print_msgs(debug, ['Proof size: ', FoundSize]),
	 print_msgs(info,  ['Time: ', Duration, ' seconds'])

	 ;

	 get_tabulation_size(TabSize),
	 TabSize > TabSize0,
	 mcall_(Cal, Goal)
	).

%------------------------------------------------------------------------
% mcall_(Calculus, ModalFormula, StatusIn, StatusOut)

mcall_(_, true, S, S) :- !. 
mcall_(_, _ : true, S, S) :- !. 

mcall_(_, F, _, _) :-
	(var(F); (F = _ : X, var(X))), !, 
	print_msgs(error, 'Encountered a variable standing for an atom to be solved.'),
	fail.	

mcall_(Cal, M1 : M2 : F, Si, So) :- !, 
	append(M1, M2, M), 
	mcall_(Cal, M : F, Si, So).

mcall_(Cal, M : F, Si, So) :- 
	functor(F, Pred, Arity),
	is_pred_of_kind(modal, Pred/Arity), !, 
	solve_modal_atom(Cal, M : F, Si, So).

mcall_(Cal, _ : F, Si, So) :- 
	functor(F, Pred, Arity),
	is_pred_of_kind(dum, Pred/Arity), !, 
	(solve_dum_atom(Cal, F, Si, So);
	 solve_dum_atom_not_in_cal(Cal, F, Si, So)).

mcall_(Cal, M : (F1, F2), Si, So) :- !, 
	mcall_(Cal, M : F1, Si, S2), 
	mcall_(Cal, M : F2, S2, So).

mcall_(Cal, M : (F1; F2), Si, So) :- !, % not complete
	mcall_(Cal, M : F1, Si, So); mcall_(Cal, M : F2, Si, So).

mcall_(Cal, M : (\+ F), S, S) :- !, % not always sound
	make_dual_modality(Cal, M, M2), 
	\+ mcall_(Cal, M2 : F, S, _).

mcall_(Cal, M : (F1 -> F2), Si, So) :- % not complete
	mcall_(Cal, M : F1, Si, So) -> mcall_(Cal, M : F2, Si, So).

mcall_(_, _ : F, S, S) :- !, F.

mcall_(Cal, F, Si, So) :- 
	functor(F, Pred, Arity),
	is_pred_of_kind(modal, Pred/Arity), !, 
	solve_modal_atom(Cal, [] : F, Si, So).

mcall_(Cal, F, Si, So) :- 
	functor(F, Pred, Arity),
	is_pred_of_kind(dum, Pred/Arity), !, 
	(solve_dum_atom(Cal, F, Si, So);
	 solve_dum_atom_not_in_cal(Cal, F, Si, So)).

mcall_(Cal, (F1, F2), Si, So) :- !, 
	mcall_(Cal, F1, Si, S2), 
	mcall_(Cal, F2, S2, So).

mcall_(Cal, (F1; F2), Si, So) :- !, 
	mcall_(Cal, F1, Si, So); mcall_(Cal, F2, Si, So).

mcall_(Cal, \+ F, S, S) :- !, % not always sound
	\+ mcall_(Cal, F, S, _).

mcall_(Cal, (F1 -> F2), Si, So) :- 
	mcall_(Cal, F1, Si, So) -> mcall_(Cal, F2, Si, So).

mcall_(_, F, S, S) :- F.

%------------------------------------------------------------------------
% solve_dum_atom_not_in_cal(Calculus, Atom, StatusIn, StatusOut)
% solve_dum_atom(Calculus, Atom, StatusIn, StatusOut)

solve_dum_atom_not_in_cal(Cal, Atom, Si, So) :-
	(get_option(priority_list_of_calculi, Calculi) -> true; 
	 get_list_of_calculi(Calculi)),  
	member(Cal2, Calculi), Cal2 \= Cal, 
	solve_dum_atom(Cal2, Atom, Si, So). 

solve_dum_atom(Cal, Atom, Si, So) :-
	get_pred(Atom, Pred), 
	get_list_of_mclauses(Cal, Pred, CNames),
	member(Name, CNames),
	get_mclause(Name, [], Atom, Body, BodyLength),
	update_status(Si, size/BodyLength, S2), 
	mcall_(Cal, Body, S2, So).          

%------------------------------------------------------------------------
% solve_modal_atom(Calculus, AtomIn, StatusIn, StatusOut) 

solve_modal_atom(Cal, M : A, Si, So) :- 
	get_option(limit_modality_length, Cal, MaxLen),
	length(M, L), L =< MaxLen, 
	get_option(limit_term_depth, MaxTermDepth),  
	termDepth(A, TD), TD =< MaxTermDepth, 
	ground_form_ma(M : A, Ag), 
	solve_modal_atom(Cal, M : A, Ag, Si, So). 

solve_modal_atom(Cal, M : A, Ag, Si, So) :- 
	(locally_completed(Ag, Cal); called(Ag, Cal)), !, 
	tabled(Ag, Cal, _, M : A), 
	update_status(Si, size/1, So). 

solve_modal_atom(Cal, M : A, Ag, Si, So) :- 
	assertz(called(Ag, Cal)),

	solve_using_mrules(Cal, (pre_rSat), M : A, A2, F2, Si, S2), 
	solve_using_mrules(Cal, (rSat), A2, A3, F3, S2, S3), 
	solve_using_mrules(Cal, (post_rSat), A3, A4, F4, S3, S4), 
	(S4 \= Si -> 
		update_status(S4, total_rSat/1, S4_), 
		update_status(S4_, size/1, S4p); 
	 S4p = S4), 

	solve_using_mrules(Cal, (rNF), A4, A5, F5, S4p, S5), 
	(S5 \= S4p -> 
		update_status(S5, total_rNF/1, S5_), 
		update_status(S5_, size/1, S5p); 
	 S5p = S5), 

	solve_using_mclauses(Cal, A5, S5p, So), 
	F5, F4, F3, F2,
	table_answer(Cal, Ag, M : A). 

solve_modal_atom(Cal, _, Ag, _, _) :- 
	retractall(called(Ag, Cal)),
	assertz(locally_completed(Ag, Cal)), 
	fail.

%------------------------------------------------------------------------
% solve_using_mrules(Calculus, Category, AtomIn, AtomOut, RemainingFormula,
%                    StatusIn, StatusOut) 

solve_using_mrules(_, _, _ : true, [] : true, true, S, S) :- !.

solve_using_mrules(Cal, Cat, Ai, Ao, RF, Si, So) :- 
	((Cat == (pre_rSat)); (Cat == (post_rSat))), 
	init_calling_history(Cat, Cal, Ai),
	get_list_of_mrules(Cal, Cat, RNames), 
	member(Name, RNames), 
	get_mrule2(Name, Ai, PreCond, Ao, RF), 
	PreCond,  
	update_status(Si, total_rSat_steps/1, So), 
	log_called_rule(Cal, Cat, Name), !. 

solve_using_mrules(_, Cat, Ai, Ai, true, S, S) :- 
	((Cat == (pre_rSat)); (Cat == (post_rSat))), !.

solve_using_mrules(_, Cat, Ai, Ai, true, S, S) :- 
	((Cat == (rSat)); (Cat == (rNF))).
	   
solve_using_mrules(Cal, Cat, Ai, Ao, RF, Si, So) :- 
	((Cat == (rSat)); (Cat == (rNF))), !, 
	init_calling_history(Cat, Cal, Ai), 
	((Cat == (rSat)) -> Option = limit_rSat; Option = limit_rNF), 
	get_option(Option, Cal, Limit),
	solve_using_mrules(Cal, Cat, Ai, Ao, RF, 1, Limit, Si, So).

solve_using_mrules(Cal, Cat, Ai, Ao, RF, Step, Limit, Si, So) :- 
	get_list_of_mrules(Cal, Cat, RNames0),
	(get_option(random_selection_of_rules, false) ->
	   RNames = RNames0;
	   slight_random_permutation(RNames0, RNames)), 
	member(RName, RNames),
	solve_using_mrule(Cal, Cat, RName, Ai, A2, F2, Si, S2), 
	((Ao = A2, RF = F2, So = S2);
	 (Step < Limit, 
	  NextStep is Step+1,
	  solve_using_mrules(Cal, Cat, A2, Ao, F3, NextStep, Limit, S2, So),  
	  (F3 = true -> RF = F2;
	   F2 = true -> RF = F3;
	   RF = (F3, F2)))). 

solve_using_mrule(Cal, Cat, RName, Ai, Ao, RF, Si, So) :- 
	(get_option(debug, false) -> true; 
	 get_info(number_of_trials, N),
	 ((N mod 1000 =:= 0, N > 0) -> 
	       print_msgs(debug, ['Trials of rules: ', N]); true), 
	 N2 is N+1,
	 set_info(number_of_trials, N2)), 

	get_mrule2(RName, Ai, PreCond, Ao, RF), 
	PreCond,
	((Cat = (rSat)) ->
	   update_status(Si, total_rSat_steps/1, So); 
	   update_status(Si, total_rNF_steps/1, So)), 
	log_called_rule(Cal, Cat, RName).

%------------------------------------------------------------------------
% solve_using_mclauses(Calculus, AtomIn, StatusIn, StatusOut)

solve_using_mclauses(Cal, A, Si, So) :-  
	get_pred(A, Pred),  
	get_list_of_mclauses(Cal, Pred, CNames), 

	member(Name, CNames), 
	get_mclause(Name, Context, Head, Body, BodyLength),

	forward_labeled_form(Cal, Head, A2),  
	break_modal_atom(A, M3, A3), 
	break_modal_atom(A2, M4, A3),  
	length(M3, M3Len),
	length(M4, M4Len), 

	% A = M5 : M6 : A3, |M6| = |M4|
	M3Len >= M4Len,
	M5Len is M3Len - M4Len,
	break_modality(M3, M5Len, M5, M6), 
	make_blf_modality(Cal, M5, M7), % blf: box lifting form

	(get_option(check_in_two_steps, Cal, true) -> 
	   (pre_check_normal_labeled_form(Cal, M5),
	    pre_check_context_instance(Cal, M7, Context),
	    pre_check_modality_instance(Cal, M6, M4));
	   (normal_labeled_form(Cal, M5),
	    context_instance(Cal, M7, Context),
	    modality_instance(Cal, M6, M4))),

	update_status(Si, size/BodyLength, S2), 
	mcall_(Cal, M5 : Body, S2, So), 
	
	(get_option(check_in_two_steps, Cal, true) -> 
	   post_check_modality_instance(Cal, M6, M4),
	   post_check_context_instance(Cal, M7, Context),
	   post_check_normal_labeled_form(Cal, M5);
	 true).  

%------------------------------------------------------------------------
% update_status(StatusIn, X/Y, StatusOut)
%
% Parameters StatusIn and StatusOut are lists of the form
% [size/_, total_rSat/_, total_rSat_steps/_, total_rNF/_, total_rNF_steps/_]
% and Y is a value for increasing the attribute X.
% Fails if limits are exceeded(?). 

update_status(StatusIn, X/Y, StatusOut) :- 
	(X \= size, get_option(debug, false)) -> StatusOut = StatusIn; 

	StatusIn = [size/W1, total_rSat/W2, total_rSat_steps/W3, 
			total_rNF/W4, total_rNF_steps/W5], 

	(X == size             -> U1 is W1 + Y; U1 = W1), 
	(X == total_rSat       -> U2 is W2 + Y; U2 = W2),
	(X == total_rSat_steps -> U3 is W3 + Y; U3 = W3),
	(X == total_rNF        -> U4 is W4 + Y; U4 = W4),
	(X == total_rNF_steps  -> U5 is W5 + Y; U5 = W5),

	get_option(max_proof_size, MaxProofSize),
	U1 =< MaxProofSize, 

	get_option(max_rule_applications, MaxRuleApplications),
	U3 + U5 =< MaxRuleApplications, 

	StatusOut = [size/U1, total_rSat/U2, total_rSat_steps/U3, 
			total_rNF/U4, total_rNF_steps/U5]. 
	
%------------------------------------------------------------------------

:- retract(portray_message(warning, clauses_not_together(_))).
