%------------------------------------------------------------------------
% tracer.pl
%
% version 2.0, 2003-2008
%
% Linh Anh Nguyen 
% Institute of Informatics
% Warsaw University
% nguyen@mimuw.edu.pl
%
%------------------------------------------------------------------------

write_trace_info(Type, N, Goal) :- 
	get_option(mtrace, true) -> 
	   nl, format('~t~d~8|  ', [N]), 
	   write(Type), write(': '), 
	   (Goal \== nil -> write(Goal); write('...')),  
	   telling(File),
	   (File == user -> write(' ?'), get_code(user, _); true);
	true.

mtrace(F) :- 
	(get_info(mtrace_step, N0) -> true; N0 = 0),
	N is N0 + 1,
	set_info(mtrace_step, N),
	write_trace_info(call, N, F), 
	mtrace(N, F).

mtrace(N, F) :- F, write_exit_or_redo_info(N, F). 
mtrace(N, F) :- write_trace_info(fail, N, F), fail.
	
write_exit_or_redo_info(N, F) :- write_trace_info(exit, N, F).
write_exit_or_redo_info(N, _) :- write_trace_info(redo, N, nil), fail.

check_mtrace((get_info(already_called_mtrace, true) ->
	        (set_info(already_called_mtrace, false), fail);
	      set_info(already_called_mtrace, true))).
	
%------------------------------------------------------------------------

mcall_(X1, X2, X3, X4) :- 
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(mcall_(X1, X2, X3, X4)).

solve_dum_atom(X1, X2, X3, X4) :- 
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_dum_atom(X1, X2, X3, X4)).

solve_modal_atom(X1, X2, X3, X4) :- 
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_modal_atom(X1, X2, X3, X4)).

solve_using_mrules(X1, X2, X3, X4, X5, X6, X7) :-
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_using_mrules(X1, X2, X3, X4, X5, X6, X7)).

solve_using_mrules(X1, X2, X3, X4, X5, X6, X7, X8, X9) :-
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_using_mrules(X1, X2, X3, X4, X5, X6, X7, X8, X9)).

solve_using_mrule(X1, X2, X3, X4, X5, X6, X7, X8) :- 
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_using_mrule(X1, X2, X3, X4, X5, X6, X7, X8)).

solve_using_mclauses(X1, X2, X3, X4) :-  
	(get_option(mtrace, false) -> fail; true), 
	check_mtrace(Check), Check, !, 
	mtrace(solve_using_mclauses(X1, X2, X3, X4)).

