/*****************************************************************************
THE WISE MEN PUZZLE 

A king wishes to know whether his three advisors (A, B, C) are as wise as they
claim to be. Three chairs are lined up, all facing the same direction, with
one behind the other. The wise men are instructed to sit down in the order
A, B, C. Each of the men can see the backs of the men sitting before them
(e.g. C can see A and B). The kings informs the wise men that he has three
cards, all of which are either black or white, at least one of which is white.
He places one card, face up, behind each of the three wise men, explaning that
each wise man determine the color of his own card. Each wise man must announce
the color of his own card as soon as he knows what it is. All know that this
will happen. The room is silent; then, after several miniutes, wise man A says
'My card is white!'.
*****************************************************************************/

% Denote the men by a, b, c, and the groups by gAB, gAC, gBC, gABC.
% We do not use lists to represent groups in order to simplify notations,
% but when there are more agents it is better to use lists.

singleton_group(G) :- G = a; G = b; G = c.

subgroup(a, G) :- G = gAB; G = gAC; G = gABC.
subgroup(b, G) :- G = gAB; G = gBC; G = gABC.
subgroup(c, G) :- G = gAC; G = gBC; G = gABC.
subgroup(G, gABC) :- (G = gAB; G = gAC; G = gBC).

union_group(G, G, G) :- !.
union_group(a, b, gAB) :- !.
union_group(b, a, gAB) :- !.
union_group(a, c, gAC) :- !.
union_group(c, a, gAC) :- !.
union_group(b, c, gBC) :- !.
union_group(c, b, gBC) :- !.
union_group(G1, G2, G2) :- subgroup(G1, G2), !.
union_group(G1, G2, G1) :- subgroup(G2, G1), !.
union_group(G1, G2, gABC).

%-----------------------------------------------------------------------------

:- calculus cKD4Ig5a.

% If Y sits behinds X then X's card is white if Y considers this as possible.

[bel(gABC)]: (white(X) :- 
		member(X, [a,b,c]), member(Y, [a,b,c]), 
		X @< Y, [pos(Y)]:white(X)).

% The following formula is "dual" to the above formula.

[bel(gABC)]: ([bel(X)]:black(Y) :- 
		member(X, [a,b,c]), member(Y, [a,b,c]), 
		Y @< X, black(Y)).

% At least one of the wise men has a white card.

[bel(gABC)]: (white(a) :- black(b), black(c)).
[bel(gABC)]: (white(b) :- black(c), black(a)).
[bel(gABC)]: (white(c) :- black(a), black(b)).

% Each of B and C does not know the color of his own card. 
% In particular, each of the men considers that it is possible that
% his own card is black.

[bel(gABC),pos(c)]:black(c).
[bel(gABC),pos(b)]:black(b).

% Change some default options:
:- set_option(limit_modality_length, cKD4Ig5a, 3).
:- set_option(limit_rSat, cKD4Ig5a, 1).
:- set_option(current_calculus, cKD4Ig5a).

% Does A believe that his card is white? 
% ?- mcall([bel(a)]:white(a)).

% ?- mcall([bel(X)]:white(X)).
% ?- mcall([bel(X)]:white(a)).
% ?- mcall([bel(a)]:white(X)).
% ?- mcall([bel(X)]:white(Y)).

%-----------------------------------------------------------------------------

